@@ -28,7 +28,7 @@ from .types import ParamType, File, Path, Choice, IntRange, STRING, INT, \
      FLOAT, BOOL, UUID
 
 # Utilities
-from .utils import echo, get_binary_stream, get_text_stream, \
+from .utils import echo, get_binary_stream, get_text_stream, open_file, \
      format_filename, get_app_dir
 
 # Terminal functions
@@ -62,8 +62,8 @@ __all__ = [
     'FLOAT', 'BOOL', 'UUID',
 
     # Utilities
-    'echo', 'get_binary_stream', 'get_text_stream', 'format_filename',
-    'get_app_dir',
+    'echo', 'get_binary_stream', 'get_text_stream', 'open_file',
+    'format_filename', 'get_app_dir',
 
     # Terminal functions
     'prompt', 'confirm', 'get_terminal_size', 'echo_via_pager',
@@ -82,4 +82,4 @@ __all__ = [
 ]
 
 
-__version__ = '2.6'
+__version__ = '3.0'
