@@ -3,6 +3,7 @@ import sys
 import codecs
 from contextlib import contextmanager
 from itertools import chain, repeat
+from functools import update_wrapper
 
 from .types import convert_type, IntRange, BOOL
 from .utils import make_str, make_default_short_help, echo
@@ -16,6 +17,10 @@ from ._compat import PY2, isidentifier, iteritems
 _missing = object()
 
 
+SUBCOMMAND_METAVAR = 'COMMAND [ARGS]...'
+SUBCOMMANDS_METAVAR = 'COMMAND1 [ARGS]... [COMMAND2 [ARGS]...]...'
+
+
 def _bashcomplete(cmd, prog_name, complete_var=None):
     """Internal handler for the bash completion support."""
     if complete_var is None:
@@ -24,7 +29,7 @@ def _bashcomplete(cmd, prog_name, complete_var=None):
     if not complete_instr:
         return
 
-    from click._bashcomplete import bashcomplete
+    from ._bashcomplete import bashcomplete
     if bashcomplete(cmd, prog_name, complete_var, complete_instr):
         sys.exit(1)
 
@@ -38,12 +43,12 @@ def invoke_param_callback(callback, ctx, param, value):
     args = getattr(code, 'co_argcount', 3)
 
     if args < 3:
-        # This will become a warning in click 3.0:
-        ## from warnings import warn
-        ## warn(Warning('Invoked legacy parameter callback "%s".  The new '
-        ##              'signature for such callbacks starting with '
-        ##              'click 2.0 is (ctx, param, value).'
-        ##              % callback), stacklevel=3)
+        # This will become a warning in Click 3.0:
+        from warnings import warn
+        warn(Warning('Invoked legacy parameter callback "%s".  The new '
+                     'signature for such callbacks starting with '
+                     'click 2.0 is (ctx, param, value).'
+                     % callback), stacklevel=3)
         return callback(ctx, value)
     return callback(ctx, param, value)
 
@@ -98,6 +103,10 @@ class Context(object):
        Added the `resilient_parsing`, `help_option_names`,
        `token_normalize_func` parameters.
 
+    .. versionadded:: 3.0
+       Added the `allow_extra_args` and `allow_interspersed_args`
+       parameters.
+
     :param command: the command class for this context.
     :param parent: the parent context.
     :param info_name: the info name for this invokation.  Generally this
@@ -117,10 +126,17 @@ class Context(object):
                            inherit from parent context.  If no context
                            defines the terminal width then auto
                            detection will be applied.
-    :param resilient_parsing: if this flag is enabled then click will
+    :param resilient_parsing: if this flag is enabled then Click will
                               parse without any interactivity or callback
                               invocation.  This is useful for implementing
                               things such as completion support.
+    :param allow_extra_args: if this is set to `True` then extra arguments
+                             at the end will not raise an error and will be
+                             kept on the context.  The default is to inherit
+                             from the command.
+    :param allow_interspersed_args: if this is set to `False` then options
+                                    and arguments cannot be mixed.  The
+                                    default is to inherit from the command.
     :param help_option_names: optionally a list of strings that define how
                               the default help parameter is named.  The
                               default is ``['--help']``.
@@ -133,6 +149,7 @@ class Context(object):
     def __init__(self, command, parent=None, info_name=None, obj=None,
                  auto_envvar_prefix=None, default_map=None,
                  terminal_width=None, resilient_parsing=False,
+                 allow_extra_args=None, allow_interspersed_args=None,
                  help_option_names=None, token_normalize_func=None):
         #: the parent context or `None` if none exists.
         self.parent = parent
@@ -145,12 +162,6 @@ class Context(object):
         self.params = {}
         #: the leftover arguments.
         self.args = []
-        #: this flag indicates if a subcommand is going to be executed.
-        #: a group callback can use this information to figure out if it's
-        #: being executed directly or because the execution flow passes
-        #: onwards to a subcommand.  By default it's `None`, but it can be
-        #: the name of the subcommand to execute.
-        self.invoked_subcommand = None
         if obj is None and parent is not None:
             obj = parent.obj
         #: the user object stored.
@@ -162,11 +173,36 @@ class Context(object):
             default_map = parent.default_map.get(info_name)
         self.default_map = default_map
 
+        #: This is similar to :attr:`invoked_subcommand` but it holds a
+        #: list of multiple subcommand names.  This is useful if multi command
+        #: chaining is enabled.
+        #:
+        #: This property and :attr:`invoked_subcommand` are synchronized.
+        #:
+        #: .. versionadded:: 3.0
+        self.invoked_subcommands = []
+
         if terminal_width is None and parent is not None:
             terminal_width = parent.terminal_width
         #: The width of the terminal (None is autodetection).
         self.terminal_width = terminal_width
 
+        if allow_extra_args is None:
+            allow_extra_args = command.allow_extra_args
+        #: Indicates if the context allows extra args or if it should
+        #: fail on parsing.
+        #:
+        #: .. versionadded:: 3.0
+        self.allow_extra_args = allow_extra_args
+
+        if allow_interspersed_args is None:
+            allow_interspersed_args = command.allow_interspersed_args
+        #: Indicates if the context allows mixing of arguments and
+        #: options or not.
+        #:
+        #: .. versionadded:: 3.0
+        self.allow_interspersed_args = allow_interspersed_args
+
         if help_option_names is None:
             if parent is not None:
                 help_option_names = parent.help_option_names
@@ -183,7 +219,7 @@ class Context(object):
         #: options, choices, commands etc.
         self.token_normalize_func = token_normalize_func
 
-        #: Indicates if resilient parsing is enabled.  In that case click
+        #: Indicates if resilient parsing is enabled.  In that case Click
         #: will do its best to not cause any failures.
         self.resilient_parsing = resilient_parsing
 
@@ -201,12 +237,39 @@ class Context(object):
         self.auto_envvar_prefix = auto_envvar_prefix
 
         self._close_callbacks = []
+        self._depth = 0
 
     def __enter__(self):
+        self._depth += 1
         return self
 
     def __exit__(self, exc_type, exc_value, tb):
-        self.close()
+        self._depth -= 1
+        if self._depth == 0:
+            self.close()
+
+    def _get_invoked_subcommand(self):
+        """This flag indicates if a subcommand is going to be executed.  A
+        group callback can use this information to figure out if it's
+        being executed directly or because the execution flow passes
+        onwards to a subcommand.  By default it's `None`, but it can be
+        the name of the subcommand to execute.
+
+        If chaining is enabled and more than one subcommand is invoked
+        then the value will be the string ``'*'``.  For chaining you
+        should be using :attr:`invoked_subcommands` instead.
+        """
+        if len(self.invoked_subcommands) == 1:
+            return self.invoked_subcommands[0]
+        if len(self.invoked_subcommands) > 1:
+            return '*'
+    def _set_invoked_subcommand(self, value):
+        if value is not None:
+            value = [value]
+        self.invoked_subcommands = value
+    invoked_subcommand = property(_get_invoked_subcommand,
+                                  _set_invoked_subcommand)
+    del _get_invoked_subcommand, _set_invoked_subcommand
 
     def make_formatter(self):
         """Creates the formatter for the help and usage output."""
@@ -348,13 +411,13 @@ class BaseCommand(object):
     """The base command implements the minimal API contract of commands.
     Most code will never use this as it does not implement a lot of useful
     functionality but it can act as the direct subclass of alternative
-    parsing methods that do not depend on the click parser.
+    parsing methods that do not depend on the Click parser.
 
-    For instance, this can be used to bridge click and other systems like
+    For instance, this can be used to bridge Click and other systems like
     argparse or docopt.
 
     Because base commands do not implement a lot of the API that other
-    parts of click take for granted, they are not supported for all
+    parts of Click take for granted, they are not supported for all
     operations.  For instance, they cannot be used with the decorators
     usually and they have no built-in callback system.
 
@@ -365,6 +428,8 @@ class BaseCommand(object):
     :param context_settings: an optional dictionary with defaults that are
                              passed to the context object.
     """
+    allow_extra_args = False
+    allow_interspersed_args = True
 
     def __init__(self, name, context_settings=None):
         #: the name the command thinks it has.  Upon registering a command
@@ -417,7 +482,8 @@ class BaseCommand(object):
         """
         raise NotImplementedError('Base commands are not invokable by default')
 
-    def main(self, args=None, prog_name=None, complete_var=None, **extra):
+    def main(self, args=None, prog_name=None, complete_var=None,
+             standalone_mode=True, **extra):
         """This is the way to invoke a script with all the bells and
         whistles as a command line application.  This will always terminate
         the application after a call.  If this is not wanted, ``SystemExit``
@@ -426,6 +492,9 @@ class BaseCommand(object):
         This method is also available by directly calling the instance of
         a :class:`Command`.
 
+        .. versionadded:: 3.0
+           Added the `standalone_mode` flag to control the standalone mode.
+
         :param args: the arguments that should be used for parsing.  If not
                      provided, ``sys.argv[1:]`` is used.
         :param prog_name: the program name that should be used.  By default
@@ -435,6 +504,15 @@ class BaseCommand(object):
                              bash completion support.  The default is
                              ``"_<prog_name>_COMPLETE"`` with prog name in
                              uppercase.
+        :param standalone_mode: the default behavior is to invoke the script
+                                in standalone mode.  Click will then
+                                handle exceptions and convert them into
+                                error messages and the function will never
+                                return but shut down the interpreter.  If
+                                this is set to `False` they will be
+                                propagated to the caller and the return
+                                value of this function is the return value
+                                of :meth:`invoke`.
         :param extra: extra keyword arguments are forwarded to the context
                       constructor.  See :class:`Context` for more information.
         """
@@ -471,15 +549,21 @@ class BaseCommand(object):
         try:
             try:
                 with self.make_context(prog_name, args, **extra) as ctx:
-                    self.invoke(ctx)
+                    rv = self.invoke(ctx)
+                    if not standalone_mode:
+                        return rv
                     ctx.exit()
             except (EOFError, KeyboardInterrupt):
                 echo(file=sys.stderr)
                 raise Abort()
             except ClickException as e:
+                if not standalone_mode:
+                    raise
                 e.show()
                 sys.exit(e.exit_code)
         except Abort:
+            if not standalone_mode:
+                raise
             echo('Aborted!', file=sys.stderr)
             sys.exit(1)
 
@@ -490,7 +574,7 @@ class BaseCommand(object):
 
 class Command(BaseCommand):
     """Commands are the basic building block of command line interfaces in
-    click.  A basic command handles command line parsing and might dispatch
+    Click.  A basic command handles command line parsing and might dispatch
     more parsing to commands nested below it.
 
     .. versionchanged:: 2.0
@@ -510,7 +594,6 @@ class Command(BaseCommand):
     :param add_help_option: by default each command registers a ``--help``
                             option.  This can be disabled by this parameter.
     """
-    allow_extra_args = False
 
     def __init__(self, name, context_settings=None, callback=None,
                  params=None, help=None, epilog=None, short_help=None,
@@ -583,6 +666,7 @@ class Command(BaseCommand):
     def make_parser(self, ctx):
         """Creates the underlying option parser for this command."""
         parser = OptionParser(ctx)
+        parser.allow_interspersed_args = ctx.allow_interspersed_args
         for param in self.get_params(ctx):
             param.add_to_parser(parser, ctx)
         return parser
@@ -644,19 +728,20 @@ class Command(BaseCommand):
                 param_order, self.get_params(ctx)):
             value, args = param.handle_parse_result(ctx, opts, args)
 
-        if args and not self.allow_extra_args and not ctx.resilient_parsing:
+        if args and not ctx.allow_extra_args and not ctx.resilient_parsing:
             ctx.fail('Got unexpected extra argument%s (%s)'
                      % (len(args) != 1 and 's' or '',
                         ' '.join(map(make_str, args))))
 
         ctx.args = args
+        return args
 
     def invoke(self, ctx):
         """Given a context, this invokes the attached callback (if it exists)
         in the right way.
         """
         if self.callback is not None:
-            ctx.invoke(self.callback, **ctx.params)
+            return ctx.invoke(self.callback, **ctx.params)
 
 
 class MultiCommand(Command):
@@ -675,23 +760,30 @@ class MultiCommand(Command):
                             passed.
     :param subcommand_metavar: the string that is used in the documentation
                                to indicate the subcommand place.
+    :param chain: if this is set to `True` chaining of multiple subcommands
+                  is enabled.  This restricts the form of commands in that
+                  they cannot have optional arguments but it allows
+                  multiple commands to be chained together.
     """
     allow_extra_args = True
+    allow_interspersed_args = False
 
     def __init__(self, name=None, invoke_without_command=False,
-                 no_args_is_help=None, subcommand_metavar='COMMAND [ARGS]...',
-                 **attrs):
+                 no_args_is_help=None, subcommand_metavar=None,
+                 chain=False, result_callback=None, **attrs):
         Command.__init__(self, name, **attrs)
         if no_args_is_help is None:
             no_args_is_help = not invoke_without_command
         self.no_args_is_help = no_args_is_help
         self.invoke_without_command = invoke_without_command
+        if subcommand_metavar is None:
+            if chain:
+                subcommand_metavar = SUBCOMMANDS_METAVAR
+            else:
+                subcommand_metavar = SUBCOMMAND_METAVAR
         self.subcommand_metavar = subcommand_metavar
-
-    def make_parser(self, ctx):
-        parser = Command.make_parser(self, ctx)
-        parser.allow_interspersed_args = False
-        return parser
+        self.chain = chain
+        self.result_callback = result_callback
 
     def collect_usage_pieces(self, ctx):
         rv = Command.collect_usage_pieces(self, ctx)
@@ -702,6 +794,43 @@ class MultiCommand(Command):
         Command.format_options(self, ctx, formatter)
         self.format_commands(ctx, formatter)
 
+    def resultcallback(self, replace=False):
+        """Adds a result callback to the chain command.  By default if a
+        result callback is already registered this will chain them but
+        this can be disabled with the `replace` parameter.  The result
+        callback is invoked with the return value of the subcommand
+        (or the list of return values from all subcommands if chaining
+        is enabled) as well as the parameters as they would be passed
+        to the main callback.
+
+        Example::
+
+            @click.group()
+            @click.option('-i', '--input', default=23)
+            def cli(input):
+                return 42
+
+            @cli.resultcallback()
+            def process_result(result, input):
+                return result + input
+
+        .. versionadded:: 3.0
+
+        :param replace: if set to `True` an already existing result
+                        callback will be removed.
+        """
+        def decorator(f):
+            old_callback = self.result_callback
+            if old_callback is None or replace:
+                self.result_callback = f
+                return f
+            def function(__value, *args, **kwargs):
+                return f(old_callback(__value, *args, **kwargs),
+                         *args, **kwargs)
+            self.result_callback = rv = update_wrapper(function, f)
+            return rv
+        return decorator
+
     def format_commands(self, ctx, formatter):
         """Extra format methods for multi methods that adds all the commands
         after the options.
@@ -727,12 +856,68 @@ class MultiCommand(Command):
         return Command.parse_args(self, ctx, args)
 
     def invoke(self, ctx):
+        def _process_result(value):
+            if self.result_callback is not None:
+                value = ctx.invoke(self.result_callback, value,
+                                   **ctx.params)
+            return value
+
         if not ctx.args:
+            # If we are invoked without command the chain flag controls
+            # how this happens.  If we are not in chain mode, the return
+            # value here is the return value of the command.
+            # If however we are in chain mode, the return value is the
+            # return value of the result processor invoked with an empty
+            # list (which means that no subcommand actually was executed).
             if self.invoke_without_command:
-                return Command.invoke(self, ctx)
+                if not self.chain:
+                    return Command.invoke(self, ctx)
+                with ctx:
+                    Command.invoke(self, ctx)
+                    return _process_result([])
             ctx.fail('Missing command.')
 
-        cmd_name = make_str(ctx.args[0])
+        args = ctx.args
+
+        # If we're not in chain mode, we only allow the invocation of a
+        # single command but we also inform the current context about the
+        # name of the command to invoke.
+        if not self.chain:
+            sub_ctx = self.handle_subcommand(ctx, args)
+            ctx.invoked_subcommands = [sub_ctx.info_name]
+
+            # Make sure the context is entered so we do not clean up
+            # resources until the result processor has worked.
+            with ctx:
+                Command.invoke(self, ctx)
+                with sub_ctx:
+                    return _process_result(sub_ctx.command.invoke(sub_ctx))
+
+        # Otherwise we make every single context and invoke them in a
+        # chain.  In that case the return value to the result processor
+        # is the list of all invoked subcommand's results.
+        contexts = []
+        while args:
+            sub_ctx = self.handle_subcommand(ctx, args, allow_extra_args=True,
+                                             allow_interspersed_args=False)
+            contexts.append(sub_ctx)
+            args = sub_ctx.args
+
+        # Now that we have all contexts, we can invoke them.
+        ctx.invoked_subcommands = [x.info_name for x in contexts]
+
+        # Make sure the context is entered so we do not clean up
+        # resources until the result processor has worked.
+        with ctx:
+            Command.invoke(self, ctx)
+            rv = []
+            for sub_ctx in contexts:
+                with sub_ctx:
+                    rv.append(sub_ctx.command.invoke(sub_ctx))
+            return _process_result(rv)
+
+    def handle_subcommand(self, ctx, args, **extra):
+        cmd_name = make_str(args[0])
         original_cmd_name = cmd_name
 
         # Get the command
@@ -755,16 +940,7 @@ class MultiCommand(Command):
                 self.parse_args(ctx, ctx.args)
             ctx.fail('No such command "%s".' % original_cmd_name)
 
-        return self.invoke_subcommand(ctx, cmd, cmd_name, ctx.args[1:])
-
-    def invoke_subcommand(self, ctx, cmd, cmd_name, args):
-        # Whenever we dispatch to a subcommand we also invoke the regular
-        # callback.  This is done so that parameters can be handled.
-        ctx.invoked_subcommand = cmd_name
-        Command.invoke(self, ctx)
-
-        with cmd.make_context(cmd_name, args, parent=ctx) as cmd_ctx:
-            return cmd.invoke(cmd_ctx)
+        return cmd.make_context(cmd_name, args[1:], parent=ctx, **extra)
 
     def get_command(self, ctx, cmd_name):
         """Given a context and a command name, this returns a
@@ -781,7 +957,7 @@ class MultiCommand(Command):
 
 class Group(MultiCommand):
     """A group allows a command to have subcommands attached.  This is the
-    most common way to implement nesting in click.
+    most common way to implement nesting in Click.
 
     :param commands: a dictionary of commands.
     """
@@ -870,7 +1046,7 @@ class Parameter(object):
 
     .. versionchanged:: 2.0
        Changed signature for parameter callback to also be passed the
-       parameter.  In click 2.0, the old callback format will still work,
+       parameter.  In Click 2.0, the old callback format will still work,
        but it will raise a warning to give you change to migrate the
        code easier.
 
@@ -886,7 +1062,7 @@ class Parameter(object):
                     without any arguments.
     :param callback: a callback that should be executed after the parameter
                      was matched.  This is called as ``fn(ctx, param,
-                     value)`` and needs to return the value.  Before click
+                     value)`` and needs to return the value.  Before Click
                      2.0, the signature was ``(ctx, value)``.
     :param nargs: the number of arguments to match.  If not ``1`` the return
                   value is a tuple instead of single value.
@@ -935,7 +1111,7 @@ class Parameter(object):
             rv = self.default()
         else:
             rv = self.default
-        return self.type(rv, self, ctx)
+        return self.type_cast_value(ctx, rv)
 
     def add_to_parser(self, parser, ctx):
         pass
@@ -948,9 +1124,9 @@ class Parameter(object):
             value = self.value_from_envvar(ctx)
         return value
 
-    def process_value(self, ctx, value):
-        """Given a value and context this runs the logic to convert the
-        value as necessary.
+    def type_cast_value(self, ctx, value):
+        """Given a value this runs it properly through the type system.
+        This automatically handles things like `nargs` and `multiple`.
         """
         def _convert(value, level):
             if level == 0:
@@ -958,6 +1134,17 @@ class Parameter(object):
             return tuple(_convert(x, level - 1) for x in value or ())
         return _convert(value, (self.nargs != 1) + bool(self.multiple))
 
+    def process_value(self, ctx, value):
+        """Given a value and context this runs the logic to convert the
+        value as necessary.
+        """
+        # If the value we were given is None we do nothing.  This way
+        # code that calls this can easily figure out if something was
+        # not provided.  Otherwise it would be converted into an empty
+        # tuple for multiple invocations which is inconvenient.
+        if value is not None:
+            return self.type_cast_value(ctx, value)
+
     def value_is_missing(self, value):
         if value is None:
             return True
@@ -1168,6 +1355,11 @@ class Option(Parameter):
                 return None, opts, secondary_opts
             raise TypeError('Could not determine name for option')
 
+        if not opts and not secondary_opts:
+            raise TypeError('No options defined but a name was passed (%s). '
+                            'Did you mean to declare an argument instead '
+                            'of an option?' % name)
+
         return name, opts, secondary_opts
 
     def add_to_parser(self, parser, ctx):
